@@ -1,10 +1,6 @@
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 /*
  * (C) 2016 by Pablo Neira Ayuso <pablo@netfilter.org>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published
- * by the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
  */
 
 #include "internal.h"
@@ -166,18 +162,6 @@ static const char *range2str(uint32_t op)
 	return expr_range_str[op];
 }
 
-static inline int nftnl_str2range(const char *op)
-{
-	if (strcmp(op, "eq") == 0)
-		return NFT_RANGE_EQ;
-	else if (strcmp(op, "neq") == 0)
-		return NFT_RANGE_NEQ;
-	else {
-		errno = EINVAL;
-		return -1;
-	}
-}
-
 static int nftnl_expr_range_snprintf(char *buf, size_t remain,
 				     uint32_t flags, const struct nftnl_expr *e)
 {
